package me.unurled.sacredrealms.cutscenes;

import org.bukkit.Bukkit;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.scheduler.BukkitTask;

/**
 * @author edasaki
 */
public class RScheduler {
    /**
     * Schedule a repeating task. Halts on plugin disable.
     * @param plugin - plugin to schedule for
     * @param r - task to run
     * @param repeatEveryTicks - ticks between each run
     */
    public static void scheduleRepeating(final JavaPlugin plugin, final Runnable r, final int repeatEveryTicks) {
        Runnable r2 = new Runnable() {
            public void run() {
                if (plugin == null || !plugin.isEnabled())
                    return;
                r.run();
                plugin.getServer().getScheduler().scheduleSyncDelayedTask(plugin, this, repeatEveryTicks);
            }
        };
        plugin.getServer().getScheduler().scheduleSyncDelayedTask(plugin, r2);
    }

    /**
     * Schedule a repeating task. Halts when halter is stopped.
     * @param plugin - plugin to schedule for
     * @param r - task to run
     * @param repeatEvery - ticks between each run
     * @param halter - halter to stop the task
     */
    public static void scheduleRepeating(final JavaPlugin plugin, final Runnable r, final int repeatEvery, final Halter halter) {
        Runnable r2 = new Runnable() {
            public void run() {
                if (plugin == null || !plugin.isEnabled() || halter.halt)
                    return;
                r.run();
                plugin.getServer().getScheduler().scheduleSyncDelayedTask(plugin, this, repeatEvery);
            }
        };
        plugin.getServer().getScheduler().scheduleSyncDelayedTask(plugin, r2);
    }

    /**
     * Used to stop repeating tasks.
     */
    public static class Halter {

        /**
         * Set this to true to stop the task.
         */
        public boolean halt = false;
    }

    /**
     * Schedule a single task.
     * @param plugin - plugin to schedule for
     * @param r - task to run
     * @param runInTicks - ticks to wait before running
     */
    public static BukkitTask schedule(JavaPlugin plugin, Runnable r, long runInTicks) {
        return Bukkit.getScheduler().runTaskLater(plugin, r, runInTicks);
    }

    /**
     * Schedule a single task to run immediately.
     * @param plugin - plugin to schedule for
     * @param r - task to run
     */
    public static void schedule(JavaPlugin plugin, Runnable r) {
        plugin.getServer().getScheduler().scheduleSyncDelayedTask(plugin, r, 0);
    }

    /**
     * Schedule an async task to run immediately.
     * @param plugin - plugin to schedule for
     * @param r - task to run
     */
    public static BukkitTask scheduleAsync(JavaPlugin plugin, Runnable r) {
        return plugin.getServer().getScheduler().runTaskAsynchronously(plugin, r);
    }

    /**
     * Schedule an async task.
     *
     * @param plugin     - plugin to schedule for
     * @param r          - task to run
     * @param runInTicks - ticks to wait before running
     * @return
     */
    public static BukkitTask scheduleAsync(JavaPlugin plugin, Runnable r, int runInTicks) {
        return plugin.getServer().getScheduler().runTaskLaterAsynchronously(plugin, r, runInTicks);
    }

    /**
     * Schedule an async task.
     * @param plugin - plugin to schedule for
     * @param r - task to run
     * @param runInTicks - ticks to wait before running
     */
    public static void scheduleAsync(JavaPlugin plugin, Runnable r, long runInTicks) {
        plugin.getServer().getScheduler().runTaskLaterAsynchronously(plugin, r, runInTicks);
    }
}
